// Import polyfills first - must be before any other imports
import './polyfills';

// Import HMR fix for production (prevents WebSocket errors)
import { initViteHmrFix } from './vite-hmr-fix';

// CRITICAL: Production setup for preventing 502 errors
// This forcefully disables Vite WebSockets and other problematic connections
if (import.meta.env.PROD) {
  console.log('🛡️ Initializing production safeguards');

  // Apply Vite HMR fix to intercept problematic connections
  initViteHmrFix();

  // Set global flags to control WebSocket behavior
  (window as any).__PRODUCTION_MODE__ = true;
  (window as any).__DISABLE_VITE_WS__ = true;
  
  // Critical: Add global error handler to catch and manage WebSocket issues
  window.addEventListener('error', (e) => {
    // Check if it's a WebSocket-related error
    const errorMessage = e.message || '';
    if (
      errorMessage.includes('WebSocket') ||
      errorMessage.includes('ws://') ||
      errorMessage.includes('wss://') ||
      (e.error && e.error.toString().includes('WebSocket'))
    ) {
      console.log('Showing WebSocket error notification instead of error screen');
      e.preventDefault();
      return false;
    }
    
    // Let other errors through to be handled normally
    return true;
  });
  
  // Provide custom notification for users in case of WebSocket errors
  if (!window.onerror) {
    window.onerror = (message, source, lineno, colno, error) => {
      if (
        (message as string).includes('WebSocket') ||
        (error && error.toString().includes('WebSocket'))
      ) {
        console.log('Suppressing WebSocket error notification');
        return true; // Prevents default handling
      }
      return false; // Allow other errors to be processed normally
    };
  }
}

// Create a safer runtime handler for WebSocket errors
// Instead of intercepting the WebSocket constructor (which has TypeScript issues),
// we'll create a utility that wraps WebSocket instances and catches errors
const safeCreateWebSocket = (url: string | URL, protocols?: string | string[]) => {
  // Check for problematic URLs that would cause errors
  const urlString = url.toString();
  
  if (urlString.includes('localhost:undefined') || 
      urlString.includes('undefined/') ||
      urlString.includes('undefined:')) {
    console.warn(`Prevented connection to problematic WebSocket URL: ${urlString}`);
    
    // Return a connected flag that the application can check
    return {
      socket: null,
      connected: false,
      error: new Error(`Invalid WebSocket URL: ${urlString}`)
    };
  }
  
  try {
    // Create the actual WebSocket
    const socket = new WebSocket(url, protocols);
    
    // Return success
    return {
      socket,
      connected: true,
      error: null
    };
  } catch (error) {
    console.warn(`Error creating WebSocket for URL ${urlString}:`, error);
    return {
      socket: null,
      connected: false,
      error
    };
  }
};

// Patch the window object to help with error diagnostics
// This doesn't replace the WebSocket constructor, but adds a utility for debugging
Object.defineProperty(window, '__safeWebSocket', {
  value: safeCreateWebSocket,
  writable: false,
  configurable: false
});

// Add global error handler immediately
window.addEventListener('error', function(event) {
  console.error('Global error handler caught:', event.error || event.message);
  
  // Skip WebSocket errors for Vite HMR (don't show error screen)
  const errorText = String(event.error || event.message);
  
  // Skip WebSocket connection errors
  if (errorText.includes('WebSocket') && 
      (errorText.includes('localhost:undefined') || 
       errorText.includes('undefined/') ||
       errorText.includes('undefined:'))) {
    console.warn('Suppressing WebSocket error screen for:', errorText);
    return false;
  }
  
  // Skip errors with hosting platform references
  if (errorText.includes('spock.replit.dev') || 
      errorText.includes('replit.dev/__replco') || 
      errorText.includes('.repl.co') || 
      errorText.includes('replit.app')) {
    console.warn('Suppressing error with platform reference');
    return false;
  }
  
  if (document.readyState === 'complete' || document.readyState === 'interactive') {
    showErrorScreen(event.error || event.message);
  }
  // Let the error propagate for dev tools
  return false;
});

window.addEventListener('unhandledrejection', function(event) {
  console.error('Unhandled promise rejection:', event.reason);
  
  // Get the reason text for checking
  const reasonText = String(event.reason || '');
  
  // Skip WebSocket errors for Vite HMR (don't show error screen)
  if (reasonText.includes('WebSocket') && 
      (reasonText.includes('localhost:undefined') || 
       reasonText.includes('undefined/') ||
       reasonText.includes('undefined:'))) {
    console.warn('Suppressing WebSocket rejection screen for:', reasonText);
    return false;
  }
  
  // Skip errors with hosting platform references
  if (reasonText.includes('spock.replit.dev') || 
      reasonText.includes('replit.dev/__replco') || 
      reasonText.includes('.repl.co') || 
      reasonText.includes('replit.app')) {
    console.warn('Suppressing promise rejection with platform reference');
    return false;
  }
  
  // Skip Vite HMR errors
  if (reasonText.includes('/@vite/client') || 
      reasonText.includes('vite') || 
      reasonText.includes('hmr')) {
    console.warn('Suppressing Vite-related promise rejection');
    return false;
  }
  
  if (document.readyState === 'complete' || document.readyState === 'interactive') {
    showErrorScreen(event.reason || 'Unhandled Promise Rejection');
  }
  // Let the error propagate for dev tools  
  return false;
});

// Immediately set the page title and meta tags
document.title = "MissionLive.org - Livestream for Good";

// Create and inject a minimal loading state that will display instantly
const showLoadingIndicator = () => {
  let rootElement = document.getElementById("root");
  if (!rootElement) {
    rootElement = document.createElement("div");
    rootElement.id = "root";
    document.body.appendChild(rootElement);
  }
  
  rootElement.innerHTML = `
    <div style="display: flex; height: 100vh; width: 100vw; justify-content: center; align-items: center; font-family: system-ui, sans-serif; flex-direction: column; background-color: #fff;">
      <div style="width: 50px; height: 50px; border: 5px solid #ccc; border-top-color: #3b82f6; border-radius: 50%; animation: spin 1s linear infinite; margin-bottom: 20px;"></div>
      <div style="font-size: 18px; font-weight: 500;">Loading MissionLive...</div>
    </div>
    <style>
      @keyframes spin { to { transform: rotate(360deg); } }
      body { 
        margin: 0;
        padding: 0;
        overflow-x: hidden;
      }
      @media (prefers-color-scheme: dark) {
        body { background-color: #1a1a1a; color: #fff; }
      }
    </style>
  `;
  return rootElement;
};

// Show loading indicator immediately
const rootElement = showLoadingIndicator();
console.log("Starting MissionLive application...");

// Clear any stale cache for critical resources to avoid potential conflicts
const clearStaleCache = async () => {
  if ('caches' in window) {
    try {
      // Only in production - don't interfere with dev caching
      if (import.meta.env.PROD) {
        const cacheKeys = await window.caches.keys();
        await Promise.all(
          cacheKeys.map(key => {
            // Only clear app-related caches, not API caches
            if (key.includes('app-') || key.includes('chunk-') || key.includes('assets-')) {
              return window.caches.delete(key);
            }
            return Promise.resolve();
          })
        );
      }
    } catch (e) {
      console.warn('Cache clearing failed:', e);
      // Non-critical error, continue loading
    }
  }
};

// Use a dynamic import for the main application code
// This allows it to be loaded in chunks separately from this
// initial bootstrap code
const loadApp = async () => {
  try {
    // Start cleaning cache (don't await)
    clearStaleCache();
    
    // Import CSS first to avoid FOUC (Flash of Unstyled Content)
    await import('./index.css');
    
    // Retry mechanism for critical imports
    const retryImport = async (importFn: () => Promise<any>, retries = 3, delay = 1000) => {
      try {
        return await importFn();
      } catch (error) {
        if (retries > 0) {
          console.warn(`Import failed, retrying... (${retries} attempts left)`);
          await new Promise(resolve => setTimeout(resolve, delay));
          return retryImport(importFn, retries - 1, delay * 1.5);
        }
        throw error;
      }
    };
    
    // Import React and App dynamically with retry logic
    const [React, ReactDOM, { default: App }] = await Promise.all([
      retryImport(() => import('react')),
      retryImport(() => import('react-dom/client')),
      retryImport(() => import('./App'))
    ]);

    // Small delay to ensure styles are applied and DOM is ready
    setTimeout(() => {
      try {
        const root = ReactDOM.createRoot(rootElement);
        root.render(
          React.createElement(
            React.StrictMode, 
            {}, 
            React.createElement(App)
          )
        );
        console.log("Application rendered successfully");
      } catch (error) {
        console.error("React rendering error:", error);
        showErrorScreen(error);
      }
    }, 200);
  } catch (error) {
    console.error("Error loading application chunks:", error);
    showErrorScreen(error);
  }
};

const showErrorScreen = (error: any) => {
  const errorMsg = error instanceof Error 
    ? `${error.name}: ${error.message}\n${error.stack || ''}`
    : String(error);
    
  // Don't show error screen if we're still loading
  if (document.readyState !== 'complete' && document.readyState !== 'interactive') {
    return;
  }
  
  // Check specific error types to handle differently
  const isWebSocketError = errorMsg.includes('WebSocket') || 
                           errorMsg.includes('wss://') || 
                           errorMsg.includes('ws://');
                           
  const isPlatformError = errorMsg.includes('spock.replit.dev') || 
                          errorMsg.includes('replit.dev/__replco') || 
                          errorMsg.includes('.repl.co') || 
                          errorMsg.includes('replit.app');
                          
  const isViteError = errorMsg.includes('/@vite/client') || 
                      errorMsg.includes('vite') || 
                      errorMsg.includes('hmr');
  
  // Suppress known platform-specific errors entirely
  if (isPlatformError || isViteError) {
    console.warn('Suppressing platform-specific error, not showing error UI');
    return;
  }
  
  // Only replace content if rootElement exists
  if (rootElement) {
    // Use a less alarming UI for WebSocket errors since the app can still function
    if (isWebSocketError) {
      console.log('Showing WebSocket error notification instead of error screen');
      
      // Don't replace the entire page for WebSocket errors, just show a notification
      const wsErrorDiv = document.createElement('div');
      wsErrorDiv.id = 'ws-error-notification';
      wsErrorDiv.style.position = 'fixed';
      wsErrorDiv.style.bottom = '20px';
      wsErrorDiv.style.right = '20px';
      wsErrorDiv.style.backgroundColor = '#fff';
      wsErrorDiv.style.boxShadow = '0 2px 10px rgba(0,0,0,0.1)';
      wsErrorDiv.style.borderRadius = '8px';
      wsErrorDiv.style.padding = '15px 20px';
      wsErrorDiv.style.maxWidth = '350px';
      wsErrorDiv.style.zIndex = '9999';
      wsErrorDiv.style.border = '1px solid #f0f0f0';
      wsErrorDiv.style.fontFamily = 'system-ui, sans-serif';
      
      wsErrorDiv.innerHTML = `
        <div style="display: flex; align-items: center; margin-bottom: 10px;">
          <div style="width: 24px; height: 24px; border-radius: 50%; background-color: #fbbf24; display: flex; align-items: center; justify-content: center; margin-right: 10px;">
            <span style="color: white; font-weight: bold;">!</span>
          </div>
          <h3 style="margin: 0; font-size: 16px;">Connection Issue</h3>
          <button onclick="this.parentNode.parentNode.remove()" style="margin-left: auto; background: none; border: none; cursor: pointer; font-size: 18px; color: #888;">&times;</button>
        </div>
        <p style="margin: 0 0 10px 0; font-size: 14px; color: #333;">Some real-time features may be limited. The app will continue to function.</p>
        <button onclick="window.location.reload(true)" style="background: #3b82f6; color: white; border: none; padding: 6px 12px; border-radius: 4px; font-size: 14px; cursor: pointer; width: 100%;">
          Reload Page
        </button>
      `;
      
      document.body.appendChild(wsErrorDiv);
      
      // Auto-remove after 10 seconds
      setTimeout(() => {
        if (document.body.contains(wsErrorDiv)) {
          wsErrorDiv.remove();
        }
      }, 10000);
      
      return;
    }
    
    // Filter out any platform-specific information from error message before displaying
    const sanitizedErrorMsg = errorMsg
      .split('\n')
      .filter(line => !line.includes('replit.dev') && !line.includes('.repl.co'))
      .join('\n');
    
    // For non-WebSocket errors, show the full error screen
    rootElement.innerHTML = `
      <div style="padding: 40px; text-align: center; font-family: system-ui, sans-serif;">
        <h1 style="color: #e11d48; margin-bottom: 20px;">Application Error</h1>
        <p style="margin-bottom: 20px; font-size: 16px;">We encountered an error while loading MissionLive.</p>
        <div style="background: #f1f5f9; padding: 15px; border-radius: 6px; text-align: left; margin: 20px 0; overflow: auto; max-height: 200px;">
          <pre>${sanitizedErrorMsg || 'Unknown error'}</pre>
        </div>
        <div style="margin: 30px 0;">
          <p>You can try the following:</p>
          <ul style="text-align: left; display: inline-block; margin: 20px auto;">
            <li>Refresh the page</li>
            <li>Clear your browser cache</li>
            <li>Try a different browser</li>
          </ul>
        </div>
        <button onclick="window.location.reload(true)" style="background: #3b82f6; color: white; border: none; padding: 10px 20px; border-radius: 6px; font-size: 16px; cursor: pointer;">
          Hard Reload Page
        </button>
      </div>
    `;
  }
};

// Start loading the app with visibility monitoring
const startApp = () => {
  // Check for page visibility to avoid loading issues when tab is inactive
  if (document.visibilityState === 'visible') {
    loadApp();
  } else {
    // Wait for page to become visible before loading
    document.addEventListener('visibilitychange', () => {
      if (document.visibilityState === 'visible' && !window.__APP_LOADED__) {
        window.__APP_LOADED__ = true;
        loadApp();
      }
    }, { once: true });
    
    // Fallback in case visibility events don't fire
    setTimeout(() => {
      if (!window.__APP_LOADED__) {
        window.__APP_LOADED__ = true;
        loadApp();
      }
    }, 2000);
  }
};

// Add this type declaration for the app loaded flag
declare global {
  interface Window {
    __APP_LOADED__?: boolean;
  }
}

// Start loading the app based on document readiness
if (document.readyState === 'loading') {
  document.addEventListener('DOMContentLoaded', startApp);
} else {
  startApp();
}
